 BEGIN
    FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'SOD_CASE_ACTION_V') LOOP
        EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM sod_case_action_v';
    END LOOP;
    FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'SOD_CASE_ACTION_V') LOOP
        EXECUTE IMMEDIATE 'DROP VIEW sod_case_action_v';
    END LOOP;

    FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'SPERMISSION_V') LOOP
        EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM spermission_v';
    END LOOP;
    FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'SPERMISSION_V') LOOP
        EXECUTE IMMEDIATE 'DROP VIEW spermission_v';
    END LOOP;

    FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'SUSER_NAME_V') LOOP
        EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM suser_name_v';
    END LOOP;
    FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'SUSER_NAME_V') LOOP
        EXECUTE IMMEDIATE 'DROP VIEW suser_name_v';
    END LOOP;
END;
/

CREATE VIEW suser_name_v AS 
	SELECT 
		unique_user_id, 
		display_name 
	FROM suser 
	WHERE effective = 1 and deleted = 0;

CREATE VIEW spermission_v AS
   SELECT DISTINCT spermission.id,
        spermission.account_grant,
        spermission.assignable,
        spermission.authoritative,
        TO_CHAR(spermission.bound_by_id) bound_by_id,
        spermission.boundable,
        spermission.cost,
        spermission.deleted,
        spermission.description,
        spermission.display_name,
        spermission.driver_id,
        spermission.effective,
        spermission.exclude_from_catalog,
        spermission.is_curated,
        spermission.logical_id,
        spermission.merged,
        spermission.name,
        spermission.not_reviewable,
        TO_CHAR(spermission.permission_id) permission_id,
        spermission.prov_attribute,
        spermission.prov_by_unique_permission_id,
        spermission.prov_native_value,
        spermission.prov_unique_driver_id,
        spermission.provisionable,
        spermission.risk,
        spermission.type,
        spermission.unique_bound_by_id,
        spermission.unique_permission_id,
        spermission.value,
        spermission.value_type,
        spermission.application,
        spermission.base_application,
        spermission.collected_entity,
        spermission.data_collection,
        spermission.effective_entity,
        spermission.sversion,
        spermission.auto_generated,
        spermission.prov_unique_permission_id,
        auth_by.brole_name authorized_by,
        LISTAGG(TO_CHAR(su.display_name), TO_CHAR(' | ')) WITHIN GROUP (ORDER BY su.display_name) perm_owner_name
     FROM spermission 
     LEFT JOIN (SELECT brole.brole_name, brole.unique_item_id 
            FROM (SELECT business_role_id, unique_item_id,(SELECT DISTINCT brole.name FROM business_role brole WHERE br_incl_item.business_role_id = brole.id AND brole.approval_state = 'PUBLISHED') brole_name
                FROM business_role_incl_item br_incl_item) brole, spermission       
            WHERE brole.unique_item_id = spermission.unique_permission_id AND brole.brole_name IS NOT NULL) auth_by ON auth_by.unique_item_id = spermission.unique_permission_id
      LEFT JOIN resolved_spermission_owner rso ON rso.permission = spermission.id
      LEFT JOIN suser_name_v su ON TO_CHAR(su.unique_user_id) = TO_CHAR(rso.owner_unique_id)
     WHERE spermission.deleted = 0 AND spermission.effective = 1
     GROUP BY spermission.id,
        spermission.account_grant,
        spermission.assignable,
        spermission.authoritative,
        TO_CHAR(spermission.bound_by_id),
        spermission.boundable,
        spermission.cost,
        spermission.deleted,
        spermission.description,
        spermission.display_name,
        spermission.driver_id,
        spermission.effective,
        spermission.exclude_from_catalog,
        spermission.is_curated,
        spermission.logical_id,
        spermission.merged,
        spermission.name,
        spermission.not_reviewable,
        TO_CHAR(spermission.permission_id),
        spermission.prov_attribute,
        spermission.prov_by_unique_permission_id,
        spermission.prov_native_value,
        spermission.prov_unique_driver_id,
        spermission.provisionable,
        spermission.risk,
        spermission.type,
        spermission.unique_bound_by_id,
        spermission.unique_permission_id,
        spermission.value,
        spermission.value_type,
        spermission.application,
        spermission.base_application,
        spermission.collected_entity,
        spermission.data_collection,
        spermission.effective_entity,
        spermission.sversion,
        spermission.auto_generated,
        spermission.prov_unique_permission_id,
        auth_by.brole_name
     ORDER BY spermission.name;

CREATE VIEW sod_case_action_v AS 
SELECT 
    sod_case_info.id case_number,
    sod_case_info.state case_status,
    sod_case_info.policy,
    sod_case_info.policy_name,
    sod_policy.deleted policy_status,
    sod_case_info.violator_is_user,
    sod_case_info.violator_name,
    sod_case_info.violator_title,
    sod_case_action.id action_id,
    sod_case_action.action,
    sod_case_action.action_comment,
    sod_case_action.action_time,
    suser.display_name performed_by,
    sod_case_action.control_period,
    CASE
         WHEN sod_case_action.control_period IS NOT NULL THEN sod_case_action.action_time + numtodsinterval(sod_case_action.control_period, 'day')
         ELSE TO_TIMESTAMP(NULL)
    END approval_expiration,
    sod_removal_item.type item_type,
    CASE
        WHEN ((sod_removal_item.type) = 'PERMISSION') THEN spermission_v.name
        ELSE role_policy.name
    END item_name,
    count(cri.id) AS total_number,
    (
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'PENDING' AND cri.provisioning_type = 'MANUAL'
	) pending_manual_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'PENDING' AND cri.provisioning_type = 'IDENTITY_MANAGER'
	) pending_idm_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'PENDING' AND cri.provisioning_type = 'EXTERNAL_WORKFLOW'
	) pending_ext_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'INITIALIZED' 
	) initialized_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'VERIFIED' 
	) verified_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'REFUSED' 
	) refused_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'VERIFICATION_IGNORED' 
	) verification_ignored_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'NOT_VERIFIED' 
	) not_verified_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'VERIFICATION_TIMEOUT' 
	) verification_timeout_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'ERROR' 
	) error_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'COMPLETED' AND cri.provisioning_type = 'MANUAL'
	) completed_manual_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'COMPLETED' AND cri.provisioning_type = 'IDENTITY_MANAGER'
	) completed_idm_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'COMPLETED' AND cri.provisioning_type = 'EXTERNAL_WORKFLOW'
	) completed_ext_number,
	(
	 SELECT COUNT(*) 
	 FROM sod_case_action sca2
	 LEFT OUTER JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
	 LEFT OUTER JOIN change_item_action cia ON cri.latest_action = cia.id
	 WHERE sca2.id = sod_case_action.id AND cia.change_item_status = 'IN_PROGRESS' 
	) in_progress_number
   FROM sod_case_info
     JOIN sod_policy ON sod_policy.id = sod_case_info.policy
     JOIN sod_case_action ON sod_case_action.sod_case = sod_case_info.id
     LEFT JOIN sod_removal_item ON sod_removal_item.sod_case_action = sod_case_action.id
     LEFT JOIN spermission_v ON spermission_v.unique_permission_id = sod_removal_item.unique_id
     LEFT JOIN role_policy ON role_policy.unique_policy_id = sod_removal_item.unique_id AND role_policy.deleted = 0
     LEFT JOIN suser ON sod_case_action.performed_by = suser.unique_user_id AND suser.deleted = 0 AND suser.effective = 1
     LEFT JOIN change_request_item cri ON cri.sod_case_action = sod_case_action.id
   GROUP BY sod_case_info.id, sod_case_info.state, sod_case_info.policy, sod_case_info.policy_name, sod_policy.deleted, sod_case_info.violator_is_user, sod_case_info.violator_name, sod_case_info.violator_title, sod_case_action.id, sod_case_action.action, sod_case_action.action_comment, sod_case_action.action_time, suser.display_name, sod_case_action.control_period, spermission_v.name, role_policy.name, sod_removal_item.type
  ORDER BY sod_case_info.policy_name, sod_case_info.policy, sod_case_info.violator_name, sod_case_action.action_time;

GRANT SELECT ON igops.suser_name_v TO igrptuser;
CREATE OR REPLACE PUBLIC SYNONYM suser_name_v for igops.suser_name_v;

GRANT SELECT ON igops.spermission_v TO igrptuser;
CREATE OR REPLACE PUBLIC SYNONYM spermission_v for igops.spermission_v;

GRANT SELECT ON igops.sod_case_action_v TO igrptuser;
CREATE OR REPLACE PUBLIC SYNONYM sod_case_action_v for igops.sod_case_action_v;
